@@ -11,7 +11,7 @@ use Moo;
 use IPC::Run3; # For run3().
 use Types::Standard qw/Any ArrayRef HasMethods HashRef Int Str/;
 
-our $VERSION = '2.64';
+our $VERSION = '2.65';
 
 my $DATA_SECTION = get_data_section; # load once
 my $DEFAULT_COMBINE = 1; # default for combine_node_and_port
@@ -626,6 +626,13 @@ sub from_graph {
 	my ($v_attr, $e_attr) = qw(get_vertex_attribute get_edge_attribute);
 	$v_attr .= '_by_id' if $is_multiv;
 	$e_attr .= '_by_id' if $is_multie;
+	my %first2edges;
+	for my $e (sort {$a->[0] cmp $b->[0] || $a->[1] cmp $b->[1]} $g->unique_edges) {
+		my @edges = $is_multie
+			? map $g->$e_attr(@$e, $_, 'graphviz') || {}, sort $g->get_multiedge_ids(@$e)
+			: $g->$e_attr(@$e, 'graphviz')||{};
+		push @{ $first2edges{$e->[0]} }, map [ from => $e->[0], to => $e->[1], %$_ ], @edges;
+	}
 	for my $v (sort $g->unique_vertices) {
 		my @vargs = $v;
 		if ($is_multiv) {
@@ -634,12 +641,7 @@ sub from_graph {
 		}
 		my $attrs = @vargs ? $g->$v_attr(@vargs, 'graphviz') || {} : {};
 		$self->add_node(name => $v, %$attrs) if keys %$attrs;
-		for my $e (sort {$a->[1] cmp $b->[1]} $g->edges_from($v)) {
-			my @edges = $is_multie
-				? map $g->$e_attr(@$e, $_, 'graphviz') || {}, sort $g->get_multiedge_ids(@$e)
-				: $g->$e_attr(@$e, 'graphviz')||{};
-			$self->add_edge(from => $v, to => $e->[1], %$_) for @edges;
-		}
+		$self->add_edge(@$_) for @{ $first2edges{$v} };
 	}
 	$self;
 }
